
import os
import json

from typing import Union
from requests import Response
from py_behrtech.Parsers.defaults import Defaults


class ModelParser(Defaults):

    def __init__(self, req: Response):

        def getCount():
            count = json.loads(self.data).get('count')
            return count if count is not None else 1

        Defaults.__init__(self)
        self.response: Response = req
        self.data: str = req.text
        self.count = getCount()
        self.models = json.loads(self.data).get('models')

    def getTypes(self) -> Union[list, dict]:
        """
        Gets type of sensor model

        :return: Type of sensor model
        """

        if self.models:
            return [{'name': x.get('name'), 'type': x.get('type')} for x in self.models]
        else:
            return {'name': json.loads(self.data).get('name'), 'type': json.loads(self.data).get('type')}

    def getComponents(self) -> Union[list, dict]:
        """
        Gets measured components for a specified sensor model

        :return: Measure components
        """

        if self.models:
            return [{'name': x.get('name'), 'components': x.get('components')} for x in self.models]
        else:
            return json.loads(self.data).get('components')

    def getComponentsNames(self) -> list:
        """
        Gets a list of measured components names for the specified sensor model

        :return: Measured components names
        """

        if self.models:
            return [{'name': x.get('name'), 'components': [y for y in x.get('components')]} for x in self.models]
        else:
            return [x for x in json.loads(self.data).get('components')]

    def getUnits(self) -> Union[list, dict]:
        """
        Gets units for measured components for the specified sensor model

        :return: Units for measured components
        """

        if self.models:
            return [{
                'name': x.get('name'),
                'units': {
                    y: x.get('components').get(y)['unit'] for y in x.get('components')
                }
            } for x in self.models]
        else:
            components = json.loads(self.data).get('components')
            return {x: components.get(x)['unit'] for x in components}




from .Calls import Azure, BaseStation, Messages, Models, Mqtt, Nodes, Plugins, System, User


class Calls(Azure, BaseStation, Messages, Models, Mqtt, Nodes, Plugins, System, User):
    """
    Class for handling API requests to BehrTech's industrial gateways
    """

    def __init__(self, username: str, password: str, server_address: str):
        Azure.__init__(self)
        BaseStation.__init__(self)
        Messages.__init__(self)
        Models.__init__(self)
        Mqtt.__init__(self)
        Nodes.__init__(self)
        Plugins.__init__(self)
        System.__init__(self)
        User.__init__(self)
        self.username = username
        self.password = password
        self.server_address = f"https://{server_address}:443"
        self.jwt_token = self.login()

    def __repr__(self):
        return f"Connection to {self.server_address} for user {self.username}"

    def set_username(self, username: str):
        """
        Sets the login username for the given gateway

        :param username: username credential
        """

        self.username = username

    def get_username(self) -> str:
        """
        Returns the username credential for the given gateway

        :return: The username credential for the given gateway
        """

        return self.username

    def set_password(self, password: str):
        """
        Sets the login password for the given gateway

        :param password: password credential
        """

        self.password = password

    def get_password(self) -> str:
        """
        Returns the password credential for the given gateway

        :return: The password credential for the given gateway
        """

        return self.password

    def set_server_address(self, server_address):
        """
        Sets the IP address for the given gateway

        :param server_address: The IP address of the given gateway
        """

        self.server_address = server_address

    def get_server_address(self) -> str:
        """
        Returns the set IP address of the gateway

        :return: The set IP address of the gateway
        """

        return self.server_address

    def test_connection(self) -> bool:
        if self.jwt_token:
            return True
        else:
            return False



import requests

from py_behrtech.Parsers.messageParser import MessageParser
from py_behrtech.exceptions import check_status_code
from py_behrtech.Calls.functions import buildParameter


class Messages:

    def __init__(self):
        self.username = None
        self.password = None
        self.server_address = None
        self.jwt_token = None

    def messagesDelete(self, deleteAll: bool = False) -> dict:
        """
        Deletes all messages from the gateways

        :param deleteAll: Verifies deletion was on purpose
        :return: Deletion status information
        """

        req = requests.delete(url=self.server_address + f"/v2/messages?deleteAll={deleteAll}", verify=False,
                              headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)

    def messagesGet(self, returnCount: int = '', offset: int = '', epEui: str = '', epName: str = '', bsEui: str = '',
                    sensorType: str = '', start: int = '', end: int = '') -> MessageParser:
        """
        Returns messages from the gateway.

        :param returnCount: Amount of messages to be requested. (-1 for all)
        :param offset: Message number to start the request from
        :param epEui: Unique epEui number of the node to be requested
        :param epName: Endpoint of which messages to be requested. Name is not unique
        :param bsEui: bsEui number of the messages to be requested
        :param sensorType: node type of node to be requested
        :param start: Date to start request from. Unix timestamp in seconds
        :param end: Date to end request from start date. Unix timestamp in seconds
        :return: Parser object of requested messages

        Example:
            Start: int(datetime.datetime(year=2020, month=1, day=1).timestamp())
            End: int(datetime.datetime(year=2020, month=2, day=1).timestamp())
        """

        req = requests.get(url=self.server_address + f"/v2/messages" + buildParameter(params=vars()), verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return MessageParser(req=req)
        else:
            check_status_code(req=req)

    def messagesPost(self):
        pass




import os


class Defaults:

    def __init__(self):
        self.response = None
        self.data = None
        self.count = None

    def getResponse(self):
        """
        Returns the response of the class

        :return: The response of the class
        """

        return self.response

    def getData(self):
        """
        Returns the data of the class

        :return: The data of the class
        """

        return self.data

    def getCount(self):
        """
        Returns the count of the class

        :return: The count of the class
        """

        return self.count

    def writeDateToFile(self, file_name: str, file_path: os.path = 'Outputs/'):
        """
        Outputs the data of the class to a JSON file

        :param file_path: The path to write the file to
        :param file_name: The name of the file for the data of the class to be outputted to
        """

        file_path = file_path if os.path.isdir(file_path) else 'Outputs/'

        text_file = open(file_path + file_name + ".json", "w")
        text_file.write(self.data)
        text_file.close()



import requests

from py_behrtech.exceptions import check_status_code
from py_behrtech.Calls.functions import buildParameter


class Azure:

    def __init__(self):
        self.username = None
        self.password = None
        self.server_address = None
        self.jwt_token = None

    def azureMappingGet(self, returnCount: int = '', offset: int = '', azureFunctionID: str = '') -> dict:
        """
        Gets information about the requested Azure mappings

        :param returnCount: The amount of Azure mappings to be requested. -1 is ALL
        :param offset: The Azure mapping count to start the Azure mapping data request from
        :param azureFunctionID: Unique Azure function ID
        :return: Detailed information about the requested Azure mappings
        """

        req = requests.get(url=self.server_address + f"/v2/azuremapping" + buildParameter(params=vars()), verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)




from py_behrtech.Parsers.messageParser import MessageParser
from py_behrtech.Parsers.nodeParser import NodeParser
from py_behrtech.Parsers.modelParser import ModelParser
from py_behrtech.Parsers.defaults import Defaults



import requests

from py_behrtech.exceptions import check_status_code
from py_behrtech.Calls.functions import buildParameter
from py_behrtech.Parsers.modelParser import ModelParser


class Models:

    def __init__(self):
        self.username = None
        self.password = None
        self.server_address = None
        self.jwt_token = None

    def sensorModelsDelete(self):
        pass

    def sensorModelsGet(self, returnCount: int = '', offset: int = '') -> ModelParser:
        """
        Gets information on all registered node models

        :param returnCount: The amount of node models to be requested. (-1 for all)
        :param offset: Node model amount to start the request from
        :return: Parser object of configured node models
        """

        req = requests.get(url=self.server_address + f"/v2/sensormodels" + buildParameter(params=vars()), verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return ModelParser(req=req)
        else:
            check_status_code(req=req)

    def sensorModelsPost(self):
        pass

    def sensorModelsSensorTypeDelete(self):
        pass

    def sensorModelsSensorTypeGet(self, sensorType: str) -> ModelParser:
        """
        Gets information on a registered node model

        :param sensorType: Unique identifier of the node model to be requested
        :return: Parser object of configured node model
        """

        req = requests.get(url=self.server_address + f"/v2/sensormodels/{sensorType}", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return ModelParser(req=req)
        else:
            check_status_code(req=req)

    def sensorModelsSensorTypePost(self):
        pass
